function [distances, success] = compute_distance_matrix(...
  image_function, number_shapes, save_filename_prefix, registration_options, convergence_error_function, desired_shape_space_dimensionality, useCurrentResults)
  % [result, success] = compute_distance_matrix(image_function, number_shapes, save_filename_prefix, registration_options, convergence_error_function, desired_shape_space_dimensionality, maximum_parallel_cpus)
  %
  %   Computes the LDDMM distance between each pair of shapes and returns a distance matrix in result.
  %   Parallel.
  %   
  %   2012-12-01 tebuck: copied from register_shapes_to_mean_shape.m.
  %   2013-04-20 tebuck: added argument  to stop when the number of distances known is the same as the number of degrees of freedom of a distance matrix for points of a specified dimensionality.
  %   2013-04-24 tebuck: adding parfor if new argument maximum_parallel_cpus is greater than one.
  %   2013-05-13 tebuck: skipped distances are now silent as those messages were mostly noise.
  %   2013-08-14 gj:     Removed skipping distance matrix if file already
  %                      exists, so we can add images and update the shape 
  %                      as preprocessed results complete
  %   2013-08-18 gj:     Distance matrix fills randomly to less bias reconstruct_distance_matrix
  %   2013-08-28 gj:     Added return statement in catch block of
  %                      compute_single_distance, and removed parfor loop
  %   2013-08-29 gj:     Added "useCurrentResults" to generate a model with
  %                      only current results, and fixed bug where .tmp
  %                      files were being left behind
  % Aug 30, 2013 G. Johnson    Changed they way files are input into the
%                            diffeomorphic model function
  % Sept 4, 2013 G. Johnson Improved error handling

  
    if ~exist('desired_shape_space_dimensionality', 'var')
    desired_shape_space_dimensionality = inf;
    end

    if ~exist('useCurrentResults', 'var')
    useCurrentResults = false;
    end

    %success = false;
    success = true;
    distances = zeros(number_shapes, number_shapes);
    distances_save_filename = [save_filename_prefix];
    % mkdir(save_filename_prefix)
    % mkdir([save_filename_prefix, 'distances'])
    mkdir(distances_save_filename)


    if useCurrentResults
        files = dir([save_filename_prefix filesep '*.mat']);
        files = {files.name};
        tokens = regexp(files, '[^0-9]', 'split');
        tokens = vertcat(tokens{:});
        xy = str2double(tokens(:,1:2));
        
        x = xy(:,1);
        y = xy(:,2);
        
        inds = 1:length(x);
        
    else 
        [y,x] = meshgrid(1:number_shapes,1:number_shapes);

        inds = find(triu(ones(number_shapes)) .* ~eye(number_shapes));

        %Randomly fill the matrix to get uniform distribution of distances when
        %using resconstruct_distance_matrix
%          inds = inds(randperm(length(inds)));
    end
    
    % Either compute distances in series or just load them and check success using this same loop:
    for i = 1:length(inds)
        shape_index = x(inds(i));
        shape_index2 = y(inds(i));
        disp(['Computing distance ( ' num2str(shape_index) ' , ' num2str(shape_index2) ')']);
        
        [distances(shape_index, shape_index2), individual_success] = compute_single_distance(image_function, save_filename_prefix, registration_options, convergence_error_function, desired_shape_space_dimensionality, shape_index, shape_index2, useCurrentResults);
        % Distance matrix is not complete if one distance is missing:
        success = success && individual_success;
    end
    
    distances = distances + distances';
    
    if success
        save([distances_save_filename, '_incomplete.mat'], 'distances', 'success');
    end
    
%   end

  % Reconstruct entries with nans:
    distances = reconstruct_distance_matrix2(distances, desired_shape_space_dimensionality);
  
  if success
    save([distances_save_filename, '.mat'], 'distances', 'success');
  end
  % success = true;

end

function [distance, success] = compute_single_distance(...
  image_function, save_filename_prefix, registration_options, convergence_error_function, desired_shape_space_dimensionality, shape_index, shape_index2, useCurrentResults)

  % Only compute the first (desired_shape_space_dimensionality + 2) columns:
%   if shape_index > (desired_shape_space_dimensionality + 2)
%     % continue
%     distance = nan;
%     % success = false;
%     success = true;
%     return
%   end

  % Check if work has been done
  imind2 = shape_index;
  imind1 = shape_index2;
  

  save_filename = [save_filename_prefix, filesep, num2str(imind2, '%d'), num2str(imind1, '_%d')]; 
  
  [can_start, final_name, final_exists, tempname] = chunk_start(save_filename);
  
  if (~can_start) || useCurrentResults
    stack = dbstack();
    % warning('Skipping "%s" at line %d in file %s.', save_filename, stack(1).line, stack(1).file)
    % fprintf('compute_distance_matrix: Skipping "%s" at line %d in file %s.\n', save_filename, stack(1).line, stack(1).file)
    if final_exists
      success = true;
      r = [];
      load([save_filename '.mat'], 'r')
      % distances(shape_index, shape_index2) = r.total_distance;
      distance = r.total_distance;
      return;
    else
      success = false;
      % warning('  Being computed elsewhere.')
      % fprintf('compute_distance_matrix: Being computed elsewhere.\n')
      % distances(shape_index, shape_index2) = nan;
      distance = NaN;
      return;
    end
  else
      
    fail = false;

    try
        [source] = image_function(shape_index); 
        [target] = image_function(shape_index2); 
    catch
        fail = true;
    end
      
    if fail | isempty(source) | isempty(target)
        disp(['Could not retreive images ' num2str(shape_index) ' and/or ' num2str(shape_index2)])
        distance = NaN;
        success = false;
    else

        fprintf('compute_distance_matrix: Computing %s\n', save_filename)
        % Get image


        % Perform work
        registration_options.convergence_registration_error = ...
            convergence_error_function(source, target);
        registration_options.single_sided = false;
        registration_options.just_compute_distance = true;
        % registration_options.verbose = 0;
        % registration_options

        [~,range] = pc12_cropImg(source+target, 5);
        
        source = source(range(1):range(2), range(3):range(4),:);
        target = target(range(1):range(2), range(3):range(4),:);
        
        imsize = size(source);
        targetsize = ceil(max([imsize(1:2)])/registration_options.window_radius) * registration_options.window_radius;
        
        padding = (targetsize - imsize(1:2))/2;
        source = padarray(source, [floor(padding) 0]);
        target = padarray(target, [floor(padding) 0]);
        
        source = padarray(source, [abs(floor(padding) - ceil(padding)) 0], 'pre');
        target = padarray(target, [abs(floor(padding) - ceil(padding)) 0], 'pre');
        
        
        
        r = Greedy3D_lambda_pre_compressed(...
          source, target, 1, registration_options)

        % Save work
        save([save_filename '.mat'], 'r')

        distance = r.total_distance;
        success = true;

    end
    chunk_finish(tempname); 
  end
  
end